@@ -3,12 +3,14 @@ package XML::Directory;
 require 5.005_62;
 use strict;
 use warnings;
+use XML::Directory::SAXGenerator;
+use XML::Directory::Exception;
 
 require Exporter;
 our @ISA = qw(Exporter);
 
 our @EXPORT_OK = qw(get_dir);
-our $VERSION = '0.41';
+our $VERSION = '0.50';
 
 
 ######################################################################
@@ -18,13 +20,17 @@ sub new {
     my ($class, $path, $details, $depth) = @_;
     $details = 2 unless @_ > 2;
     $depth = 1000 unless @_ > 3;
-    my $xml = [];
+    my $xml;
+    my $handler = '';
+    my $err_handler = '';
 
     my $self = {
 		path    => $path,
 		details => $details,
 		depth   => $depth,
 		xml     => $xml,
+		ContentHandler => $handler,
+		ErrorHanler    => $err_handler,
 		error   => 0
 	       };
     bless $self, $class;
@@ -104,17 +110,67 @@ sub get_maxdepth {
     return $self->{depth};
 }
 
-sub _set_error {
-    my $self = shift;
-    my $num = shift;
-    my $msg = shift;
-    my @err = ();
-    push @err, '<?xml version="1.0" encoding="utf-8"?>';
-    push @err, '<dirtree>';
-    push @err, "<error number=\"$num\">$msg</error>";
-    push @err, '</dirtree>';
-    $self->{error} = $num;
-    return @err;
+sub set_content_handler {
+    my ($self, $handler) = @_;
+    $handler = '' unless @_ > 1;
+    $self->{ContentHandler} = $handler;
+}
+
+sub set_error_handler {
+    my ($self, $err_handler) = @_;
+    $err_handler = '' unless @_ > 1;
+    $self->{ErrorHandler} = $err_handler;
+}
+
+sub parse_SAX {
+    my ($self, $handler, $err_handler) = @_;
+    $self->{ContentHandler} = $handler if @_ > 1;
+    $self->{ErrorHandler} = $err_handler if @_ > 2;
+    my $exception;
+
+    if ($self->{details} !~ /^[123]$/) {
+	$exception = new XML::Directory::Exception (
+		Message => "Details value ($self->{details}) invalid!",
+		Exception => undef);
+	$self->{error} = 1;
+    }
+    if ($self->{depth} !~ /^\d+$/) {
+	$exception = new XML::Directory::Exception (
+		Message => "Depth value ($self->{depth}) invalid!",
+		Exception => undef);
+	$self->{error} = 2;
+    }
+    unless (ref($self->{ContentHandler})) {
+	$exception = new XML::Directory::Exception (
+		Message => "ContentHandler is not set!",
+		Exception => undef);
+	$self->{error} = 4;
+    }
+    if ($self->{error} == 0) {
+	eval {
+	    _get_dir_SAX($self->{ContentHandler},
+			 $self->{path},
+			 $self->{details},
+			 $self->{depth});
+	};
+    }
+    if ($@) {
+	chomp $@;
+	$exception = new XML::Directory::Exception (
+		Message => "$@",
+		Exception => undef);
+	$self->{error} = 3;
+    }
+
+    if ($self->{error} > 0) {
+	$@ = $exception;
+	if (ref($self->{ErrorHandler})) {
+	    $self->{ErrorHandler}->fatal_error($exception);
+	}
+	return -1;
+    } else {
+	return 1;
+    }
 }
 
 ######################################################################
@@ -151,12 +207,12 @@ sub get_dir {
 
     foreach (<*>) {
 	if (-f $_) {
-	    __get_file($res, $_, '  ', $details);
+	    _get_file($res, $_, '  ', $details);
 	}
     }
 
     if ($depth > 0) {
-	__recursive($res,1,$path,$details,$depth); 
+	_recursive($res,1,$path,$details,$depth); 
     }
 
     push @$res, '  </directory>', '</dirtree>';
@@ -166,7 +222,20 @@ sub get_dir {
 ######################################################################
 # private procedures
 
-sub __recursive {
+sub _set_error {
+    my $self = shift;
+    my $num = shift;
+    my $msg = shift;
+    my @err = ();
+    push @err, '<?xml version="1.0" encoding="utf-8"?>';
+    push @err, '<dirtree>';
+    push @err, "<error number=\"$num\">$msg</error>";
+    push @err, '</dirtree>';
+    $self->{error} = $num;
+    return @err;
+}
+
+sub _recursive {
 
     my $res = shift;
     my $i = shift;
@@ -198,13 +267,13 @@ sub __recursive {
 
 	    foreach (<*>) {
 		if (-f $_) {
-		    __get_file($res, $_, $indent, $details);
+		    _get_file($res, $_, $indent, $details);
 		}
 	    }
 
 	    if ($depth > $i) {
 		$i++;
-		__recursive($res,$i,$path,$details,$depth); 
+		_recursive($res,$i,$path,$details,$depth); 
 		$i--; 
 	    }
 
@@ -215,7 +284,7 @@ sub __recursive {
     }
 }
 
-sub __get_file($$$$) {
+sub _get_file($$$$) {
 
 my $res     = shift;
 my $path    = shift;
@@ -224,7 +293,7 @@ my $details = shift;
 
 my @stat = stat $path;
 
-my $line = "  $indent<file name=\"$_\"";
+my $line = "  $indent<file name=\"$path\"";
 $line .= " uid=\"$stat[4]\" gid=\"$stat[5]\"" if $details > 2;
 $line .= '/' if $details == 1;
 $line .= '>';
@@ -258,17 +327,31 @@ __END__
 =head1 NAME
 
 XML::Directory - Perl extension allowing to get a content of directory 
-including sub-directories as an XML file. The current version is 0.41.
+including sub-directories as an XML file. You can generate either 
+array/string or SAX events. The current version is 0.50.
 
 =head1 SYNOPSIS
 
- $dir = new XML::Directory('/home/petr',2,5);
+ use XML::Directory;
+
+ $dir = new XML::Directory('/home/petr');
 
- $dir->set_path('/home/petr');
  $dir->set_details(3);
  $dir->set_maxdepth(10);
 
- $rc  = $dir->parse;
+---
+
+ $h = new MyHandler;
+ $e = new MyErrorHandler;
+
+ $dir->set_content_handler($h);
+ $dir->set_error_handler($e);
+
+ $rc = $dir->parse_SAX;
+
+or
+
+ $rc = $dir->parse;
 
  $res = $dir->get_arrayref;
  @res = $dir->get_array;
@@ -276,14 +359,18 @@ including sub-directories as an XML file. The current version is 0.41.
 
 or
 
- use XML::Directory(qw(get_dir));
- @xml = get_dir('/home/petr',2,5);
+ @xml = XML::Directory::get_dir('/home/petr',2,5);
 
 =head1 DESCRIPTION
 
 This utility extension provides XML::Directory class and its methods
-plus the original public function (get_dir) because of  backward 
-compatibility.
+plus the original public function (get_dir) because of backward 
+compatibility. The class methods make it possible to set parameters
+such as level of details or maximal number of nested sub-directories
+and generate either string containing the resulting XML or SAX events.
+
+The SAX generator supports both SAX1 and SAX2 handlers. There are two
+SAX interfaces available: basic ContentHandler and optional ErrorHandler.
 
 =head2 XML::DIRECTORY CLASS
 
@@ -299,6 +386,12 @@ The constructor accepts up to 3 parameters: path, details (1-3, brief or
 verbose XML) and depth (number of nested sub-directories). The last two 
 parameters are optional (defaulted to 2 and 1000).
 
+=back
+
+=head2 Methods to control parameters
+
+=over
+
 =item set_path
 
  $dir->set_path('/home/petr');
@@ -367,6 +460,30 @@ Sets or resets number of nested sub-directories to be parsed. Can also be
 set using the constructor. 0 means that only a directory specified by path 
 is parsed (no sub-directories).
 
+=item get_path
+
+ $path = $dir->get_path;
+
+Returns current path.
+
+=item get_details
+
+ $details = $dir->get_details;
+
+Returns current level of details.
+
+=item get_maxdepth
+
+ $maxdepth = $dir->get_maxdepth;
+
+Returns current number of nested sub-directories.
+
+=back
+
+=head2 Methods to generate strings
+
+=over
+
 =item parse
 
  $rc  = $dir->parse;
@@ -405,23 +522,41 @@ contains one line of the XML file).
 
 Returns a parsed XML directory image as a string.
 
-=item get_path
+=back
 
- $path = $dir->get_path;
+=head2 Methods to generate SAX events
 
-Returns current path.
+=over
 
-=item get_details
+=item set_content_handler
 
- $details = $dir->get_details;
+ $h = new MyHandler;
+ $dir->set_content_handler($h);
 
-Returns current level of details.
+Sets SAX content handler.
 
-=item get_maxdepth
+=item set_error_handler
 
- $maxdepth = $dir->get_maxdepth;
+ $e = new MyErrorHandler;
+ $dir->set_error_handler($e);
 
-Returns current number of nested sub-directories.
+Sets SAX error handler.
+
+=item $dir->parse_SAX
+
+ $rc = $dir->parse_SAX($h,$e);
+ $rc = $dir->parse_SAX($h);
+ $rc = $dir->parse_SAX;
+
+Generates SAX events for both SAX1 and SAX2 handlers. A content handler is 
+mandatory while an error handler is optional. If there is no error handler 
+registered a SAX exception is thrown. If there is an error handler available 
+an exception is thrown and an appropriate method of the error handler is 
+called.
+
+parse_SAX can either use handlers set by set_content_handler and 
+set_error_handler methods or set/reset one or both handlers itself. If there
+is no content handler registered in neither way, an exception is thrown.
 
 =back
 
